/*
 * Copyright 2013, The Sporting Exchange Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// Originally from UpdatedComponentTests/StandardTesting/REST/Rest_Post_RequestTypes_Map_SimpleMap_JSON_Volume.xls;
package com.betfair.cougar.tests.updatedcomponenttests.standardtesting.rest;

import com.betfair.testing.utils.cougar.misc.XMLHelpers;
import com.betfair.testing.utils.JSONHelpers;
import com.betfair.testing.utils.cougar.assertions.AssertionUtils;
import com.betfair.testing.utils.cougar.beans.HttpCallBean;
import com.betfair.testing.utils.cougar.beans.HttpResponseBean;
import com.betfair.testing.utils.cougar.manager.CougarManager;
import com.betfair.testing.utils.cougar.manager.RequestLogRequirement;

import org.json.JSONObject;
import org.testng.annotations.Test;
import org.w3c.dom.Document;

import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

/**
 * Ensure that when a Rest (JSON) Post operation is performed, Cougar can correctly handle a simple Map with a large volume of entries in the post body
 */
public class RestPostRequestTypesMapSimpleMapJSONVolumeTest {
    @Test
    public void doTest() throws Exception {
        // Set up the Http Call Bean to make the request
        CougarManager cougarManager1 = CougarManager.getInstance();
        HttpCallBean getNewHttpCallBean1 = cougarManager1.getNewHttpCallBean("87.248.113.14");
        cougarManager1 = cougarManager1;
        
        getNewHttpCallBean1.setOperationName("simpleMapOperation");
        
        getNewHttpCallBean1.setServiceName("baseline", "cougarBaseline");
        
        getNewHttpCallBean1.setVersion("v2");
        // Set the post body to contain a simple map with a large volume of entries
        Map map2 = new HashMap();
        map2.put("RESTJSON","{\"message\":\n           {\"simpleMap\": {\n\"1\": \"Value for 1\",\n\"2\": \"Value for 2\",\n\"3\": \"Value for 3\",\n\"4\": \"Value for 4\",\n\"5\": \"Value for 5\",\n\"6\": \"Value for 6\",\n\"7\": \"Value for 7\",\n\"8\": \"Value for 8\",\n\"9\": \"Value for 9\",\n\"10\": \"Value for 10\",\n\"11\": \"Value for 11\",\n\"12\": \"Value for 12\",\n\"13\": \"Value for 13\",\n\"14\": \"Value for 14\",\n\"15\": \"Value for 15\",\n\"16\": \"Value for 16\",\n\"17\": \"Value for 17\",\n\"18\": \"Value for 18\",\n\"19\": \"Value for 19\",\n\"20\": \"Value for 20\",\n\"21\": \"Value for 21\",\n\"22\": \"Value for 22\",\n\"23\": \"Value for 23\",\n\"24\": \"Value for 24\",\n\"25\": \"Value for 25\",\n\"26\": \"Value for 26\",\n\"27\": \"Value for 27\",\n\"28\": \"Value for 28\",\n\"29\": \"Value for 29\",\n\"30\": \"Value for 30\",\n\"31\": \"Value for 31\",\n\"32\": \"Value for 32\",\n\"33\": \"Value for 33\",\n\"34\": \"Value for 34\",\n\"35\": \"Value for 35\",\n\"36\": \"Value for 36\",\n\"37\": \"Value for 37\",\n\"38\": \"Value for 38\",\n\"39\": \"Value for 39\",\n\"40\": \"Value for 40\",\n\"41\": \"Value for 41\",\n\"42\": \"Value for 42\",\n\"43\": \"Value for 43\",\n\"44\": \"Value for 44\",\n\"45\": \"Value for 45\",\n\"46\": \"Value for 46\",\n\"47\": \"Value for 47\",\n\"48\": \"Value for 48\",\n\"49\": \"Value for 49\",\n\"50\": \"Value for 50\",\n\"51\": \"Value for 51\",\n\"52\": \"Value for 52\",\n\"53\": \"Value for 53\",\n\"54\": \"Value for 54\",\n\"55\": \"Value for 55\",\n\"56\": \"Value for 56\",\n\"57\": \"Value for 57\",\n\"58\": \"Value for 58\",\n\"59\": \"Value for 59\",\n\"60\": \"Value for 60\",\n\"61\": \"Value for 61\",\n\"62\": \"Value for 62\",\n\"63\": \"Value for 63\",\n\"64\": \"Value for 64\",\n\"65\": \"Value for 65\",\n\"66\": \"Value for 66\",\n\"67\": \"Value for 67\",\n\"68\": \"Value for 68\",\n\"69\": \"Value for 69\",\n\"70\": \"Value for 70\",\n\"71\": \"Value for 71\",\n\"72\": \"Value for 72\",\n\"73\": \"Value for 73\",\n\"74\": \"Value for 74\",\n\"75\": \"Value for 75\",\n\"76\": \"Value for 76\",\n\"77\": \"Value for 77\",\n\"78\": \"Value for 78\",\n\"79\": \"Value for 79\",\n\"80\": \"Value for 80\",\n\"81\": \"Value for 81\",\n\"82\": \"Value for 82\",\n\"83\": \"Value for 83\",\n\"84\": \"Value for 84\",\n\"85\": \"Value for 85\",\n\"86\": \"Value for 86\",\n\"87\": \"Value for 87\",\n\"88\": \"Value for 88\",\n\"89\": \"Value for 89\",\n\"90\": \"Value for 90\",\n\"91\": \"Value for 91\",\n\"92\": \"Value for 92\",\n\"93\": \"Value for 93\",\n\"94\": \"Value for 94\",\n\"95\": \"Value for 95\",\n\"96\": \"Value for 96\",\n\"97\": \"Value for 97\",\n\"98\": \"Value for 98\",\n\"99\": \"Value for 99\",\n\"100\": \"Value for 100\",\n\"101\": \"Value for 101\",\n\"102\": \"Value for 102\",\n\"103\": \"Value for 103\",\n\"104\": \"Value for 104\",\n\"105\": \"Value for 105\",\n\"106\": \"Value for 106\",\n\"107\": \"Value for 107\",\n\"108\": \"Value for 108\",\n\"109\": \"Value for 109\",\n\"110\": \"Value for 110\",\n\"111\": \"Value for 111\",\n\"112\": \"Value for 112\",\n\"113\": \"Value for 113\",\n\"114\": \"Value for 114\",\n\"115\": \"Value for 115\",\n\"116\": \"Value for 116\",\n\"117\": \"Value for 117\",\n\"118\": \"Value for 118\",\n\"119\": \"Value for 119\",\n\"120\": \"Value for 120\",\n\"121\": \"Value for 121\",\n\"122\": \"Value for 122\",\n\"123\": \"Value for 123\",\n\"124\": \"Value for 124\",\n\"125\": \"Value for 125\",\n\"126\": \"Value for 126\",\n\"127\": \"Value for 127\",\n\"128\": \"Value for 128\",\n\"129\": \"Value for 129\",\n\"130\": \"Value for 130\",\n\"131\": \"Value for 131\",\n\"132\": \"Value for 132\",\n\"133\": \"Value for 133\",\n\"134\": \"Value for 134\",\n\"135\": \"Value for 135\",\n\"136\": \"Value for 136\",\n\"137\": \"Value for 137\",\n\"138\": \"Value for 138\",\n\"139\": \"Value for 139\",\n\"140\": \"Value for 140\",\n\"141\": \"Value for 141\",\n\"142\": \"Value for 142\",\n\"143\": \"Value for 143\",\n\"144\": \"Value for 144\",\n\"145\": \"Value for 145\",\n\"146\": \"Value for 146\",\n\"147\": \"Value for 147\",\n\"148\": \"Value for 148\",\n\"149\": \"Value for 149\",\n\"150\": \"Value for 150\",\n\"151\": \"Value for 151\",\n\"152\": \"Value for 152\",\n\"153\": \"Value for 153\",\n\"154\": \"Value for 154\",\n\"155\": \"Value for 155\",\n\"156\": \"Value for 156\",\n\"157\": \"Value for 157\",\n\"158\": \"Value for 158\",\n\"159\": \"Value for 159\",\n\"160\": \"Value for 160\",\n\"161\": \"Value for 161\",\n\"162\": \"Value for 162\",\n\"163\": \"Value for 163\",\n\"164\": \"Value for 164\",\n\"165\": \"Value for 165\",\n\"166\": \"Value for 166\",\n\"167\": \"Value for 167\",\n\"168\": \"Value for 168\",\n\"169\": \"Value for 169\",\n\"170\": \"Value for 170\",\n\"171\": \"Value for 171\",\n\"172\": \"Value for 172\",\n\"173\": \"Value for 173\",\n\"174\": \"Value for 174\",\n\"175\": \"Value for 175\",\n\"176\": \"Value for 176\",\n\"177\": \"Value for 177\",\n\"178\": \"Value for 178\",\n\"179\": \"Value for 179\",\n\"180\": \"Value for 180\",\n\"181\": \"Value for 181\",\n\"182\": \"Value for 182\",\n\"183\": \"Value for 183\",\n\"184\": \"Value for 184\",\n\"185\": \"Value for 185\",\n\"186\": \"Value for 186\",\n\"187\": \"Value for 187\",\n\"188\": \"Value for 188\",\n\"189\": \"Value for 189\",\n\"190\": \"Value for 190\",\n\"191\": \"Value for 191\",\n\"192\": \"Value for 192\",\n\"193\": \"Value for 193\",\n\"194\": \"Value for 194\",\n\"195\": \"Value for 195\",\n\"196\": \"Value for 196\",\n\"197\": \"Value for 197\",\n\"198\": \"Value for 198\",\n\"199\": \"Value for 199\",\n\"200\": \"Value for 200\",\n\"201\": \"Value for 201\",\n\"202\": \"Value for 202\",\n\"203\": \"Value for 203\",\n\"204\": \"Value for 204\",\n\"205\": \"Value for 205\",\n\"206\": \"Value for 206\",\n\"207\": \"Value for 207\",\n\"208\": \"Value for 208\",\n\"209\": \"Value for 209\",\n\"210\": \"Value for 210\",\n\"211\": \"Value for 211\",\n\"212\": \"Value for 212\",\n\"213\": \"Value for 213\",\n\"214\": \"Value for 214\",\n\"215\": \"Value for 215\",\n\"216\": \"Value for 216\",\n\"217\": \"Value for 217\",\n\"218\": \"Value for 218\",\n\"219\": \"Value for 219\",\n\"220\": \"Value for 220\",\n\"221\": \"Value for 221\",\n\"222\": \"Value for 222\",\n\"223\": \"Value for 223\",\n\"224\": \"Value for 224\",\n\"225\": \"Value for 225\",\n\"226\": \"Value for 226\",\n\"227\": \"Value for 227\",\n\"228\": \"Value for 228\",\n\"229\": \"Value for 229\",\n\"230\": \"Value for 230\",\n\"231\": \"Value for 231\",\n\"232\": \"Value for 232\",\n\"233\": \"Value for 233\",\n\"234\": \"Value for 234\",\n\"235\": \"Value for 235\",\n\"236\": \"Value for 236\",\n\"237\": \"Value for 237\",\n\"238\": \"Value for 238\",\n\"239\": \"Value for 239\",\n\"240\": \"Value for 240\",\n\"241\": \"Value for 241\",\n\"242\": \"Value for 242\",\n\"243\": \"Value for 243\",\n\"244\": \"Value for 244\",\n\"245\": \"Value for 245\",\n\"246\": \"Value for 246\",\n\"247\": \"Value for 247\",\n\"248\": \"Value for 248\",\n\"249\": \"Value for 249\",\n\"250\": \"Value for 250\",\n\"251\": \"Value for 251\",\n\"252\": \"Value for 252\",\n\"253\": \"Value for 253\",\n\"254\": \"Value for 254\",\n\"255\": \"Value for 255\",\n\"256\": \"Value for 256\",\n\"257\": \"Value for 257\",\n\"258\": \"Value for 258\",\n\"259\": \"Value for 259\",\n\"260\": \"Value for 260\",\n\"261\": \"Value for 261\",\n\"262\": \"Value for 262\",\n\"263\": \"Value for 263\",\n\"264\": \"Value for 264\",\n\"265\": \"Value for 265\",\n\"266\": \"Value for 266\",\n\"267\": \"Value for 267\",\n\"268\": \"Value for 268\",\n\"269\": \"Value for 269\",\n\"270\": \"Value for 270\",\n\"271\": \"Value for 271\",\n\"272\": \"Value for 272\",\n\"273\": \"Value for 273\",\n\"274\": \"Value for 274\",\n\"275\": \"Value for 275\",\n\"276\": \"Value for 276\",\n\"277\": \"Value for 277\",\n\"278\": \"Value for 278\",\n\"279\": \"Value for 279\",\n\"280\": \"Value for 280\",\n\"281\": \"Value for 281\",\n\"282\": \"Value for 282\",\n\"283\": \"Value for 283\",\n\"284\": \"Value for 284\",\n\"285\": \"Value for 285\",\n\"286\": \"Value for 286\",\n\"287\": \"Value for 287\",\n\"288\": \"Value for 288\",\n\"289\": \"Value for 289\",\n\"290\": \"Value for 290\",\n\"291\": \"Value for 291\",\n\"292\": \"Value for 292\",\n\"293\": \"Value for 293\",\n\"294\": \"Value for 294\",\n\"295\": \"Value for 295\",\n\"296\": \"Value for 296\",\n\"297\": \"Value for 297\",\n\"298\": \"Value for 298\",\n\"299\": \"Value for 299\",\n\"300\": \"Value for 300\",\n\"301\": \"Value for 301\",\n\"302\": \"Value for 302\",\n\"303\": \"Value for 303\",\n\"304\": \"Value for 304\",\n\"305\": \"Value for 305\",\n\"306\": \"Value for 306\",\n\"307\": \"Value for 307\",\n\"308\": \"Value for 308\",\n\"309\": \"Value for 309\",\n\"310\": \"Value for 310\",\n\"311\": \"Value for 311\",\n\"312\": \"Value for 312\",\n\"313\": \"Value for 313\",\n\"314\": \"Value for 314\",\n\"315\": \"Value for 315\",\n\"316\": \"Value for 316\",\n\"317\": \"Value for 317\",\n\"318\": \"Value for 318\",\n\"319\": \"Value for 319\",\n\"320\": \"Value for 320\",\n\"321\": \"Value for 321\",\n\"322\": \"Value for 322\",\n\"323\": \"Value for 323\",\n\"324\": \"Value for 324\",\n\"325\": \"Value for 325\",\n\"326\": \"Value for 326\",\n\"327\": \"Value for 327\",\n\"328\": \"Value for 328\",\n\"329\": \"Value for 329\",\n\"330\": \"Value for 330\",\n\"331\": \"Value for 331\",\n\"332\": \"Value for 332\",\n\"333\": \"Value for 333\",\n\"334\": \"Value for 334\",\n\"335\": \"Value for 335\",\n\"336\": \"Value for 336\",\n\"337\": \"Value for 337\",\n\"338\": \"Value for 338\",\n\"339\": \"Value for 339\",\n\"340\": \"Value for 340\",\n\"341\": \"Value for 341\",\n\"342\": \"Value for 342\",\n\"343\": \"Value for 343\",\n\"344\": \"Value for 344\",\n\"345\": \"Value for 345\",\n\"346\": \"Value for 346\",\n\"347\": \"Value for 347\",\n\"348\": \"Value for 348\",\n\"349\": \"Value for 349\",\n\"350\": \"Value for 350\",\n\"351\": \"Value for 351\",\n\"352\": \"Value for 352\",\n\"353\": \"Value for 353\",\n\"354\": \"Value for 354\",\n\"355\": \"Value for 355\",\n\"356\": \"Value for 356\",\n\"357\": \"Value for 357\",\n\"358\": \"Value for 358\",\n\"359\": \"Value for 359\",\n\"360\": \"Value for 360\",\n\"361\": \"Value for 361\",\n\"362\": \"Value for 362\",\n\"363\": \"Value for 363\",\n\"364\": \"Value for 364\",\n\"365\": \"Value for 365\",\n\"366\": \"Value for 366\",\n\"367\": \"Value for 367\",\n\"368\": \"Value for 368\",\n\"369\": \"Value for 369\",\n\"370\": \"Value for 370\",\n\"371\": \"Value for 371\",\n\"372\": \"Value for 372\",\n\"373\": \"Value for 373\",\n\"374\": \"Value for 374\",\n\"375\": \"Value for 375\",\n\"376\": \"Value for 376\",\n\"377\": \"Value for 377\",\n\"378\": \"Value for 378\",\n\"379\": \"Value for 379\",\n\"380\": \"Value for 380\",\n\"381\": \"Value for 381\",\n\"382\": \"Value for 382\",\n\"383\": \"Value for 383\",\n\"384\": \"Value for 384\",\n\"385\": \"Value for 385\",\n\"386\": \"Value for 386\",\n\"387\": \"Value for 387\",\n\"388\": \"Value for 388\",\n\"389\": \"Value for 389\",\n\"390\": \"Value for 390\",\n\"391\": \"Value for 391\",\n\"392\": \"Value for 392\",\n\"393\": \"Value for 393\",\n\"394\": \"Value for 394\",\n\"395\": \"Value for 395\",\n\"396\": \"Value for 396\",\n\"397\": \"Value for 397\",\n\"398\": \"Value for 398\",\n\"399\": \"Value for 399\",\n\"400\": \"Value for 400\",\n\"401\": \"Value for 401\",\n\"402\": \"Value for 402\",\n\"403\": \"Value for 403\",\n\"404\": \"Value for 404\",\n\"405\": \"Value for 405\",\n\"406\": \"Value for 406\",\n\"407\": \"Value for 407\",\n\"408\": \"Value for 408\",\n\"409\": \"Value for 409\",\n\"410\": \"Value for 410\",\n\"411\": \"Value for 411\",\n\"412\": \"Value for 412\",\n\"413\": \"Value for 413\",\n\"414\": \"Value for 414\",\n\"415\": \"Value for 415\",\n\"416\": \"Value for 416\",\n\"417\": \"Value for 417\",\n\"418\": \"Value for 418\",\n\"419\": \"Value for 419\",\n\"420\": \"Value for 420\",\n\"421\": \"Value for 421\",\n\"422\": \"Value for 422\",\n\"423\": \"Value for 423\",\n\"424\": \"Value for 424\",\n\"425\": \"Value for 425\",\n\"426\": \"Value for 426\",\n\"427\": \"Value for 427\",\n\"428\": \"Value for 428\",\n\"429\": \"Value for 429\",\n\"430\": \"Value for 430\",\n\"431\": \"Value for 431\",\n\"432\": \"Value for 432\",\n\"433\": \"Value for 433\",\n\"434\": \"Value for 434\",\n\"435\": \"Value for 435\",\n\"436\": \"Value for 436\",\n\"437\": \"Value for 437\",\n\"438\": \"Value for 438\",\n\"439\": \"Value for 439\",\n\"440\": \"Value for 440\",\n\"441\": \"Value for 441\",\n\"442\": \"Value for 442\",\n\"443\": \"Value for 443\",\n\"444\": \"Value for 444\",\n\"445\": \"Value for 445\",\n\"446\": \"Value for 446\",\n\"447\": \"Value for 447\",\n\"448\": \"Value for 448\",\n\"449\": \"Value for 449\",\n\"450\": \"Value for 450\",\n\"451\": \"Value for 451\",\n\"452\": \"Value for 452\",\n\"453\": \"Value for 453\",\n\"454\": \"Value for 454\",\n\"455\": \"Value for 455\",\n\"456\": \"Value for 456\",\n\"457\": \"Value for 457\",\n\"458\": \"Value for 458\",\n\"459\": \"Value for 459\",\n\"460\": \"Value for 460\",\n\"461\": \"Value for 461\",\n\"462\": \"Value for 462\",\n\"463\": \"Value for 463\",\n\"464\": \"Value for 464\",\n\"465\": \"Value for 465\",\n\"466\": \"Value for 466\",\n\"467\": \"Value for 467\",\n\"468\": \"Value for 468\",\n\"469\": \"Value for 469\",\n\"470\": \"Value for 470\",\n\"471\": \"Value for 471\",\n\"472\": \"Value for 472\",\n\"473\": \"Value for 473\",\n\"474\": \"Value for 474\",\n\"475\": \"Value for 475\",\n\"476\": \"Value for 476\",\n\"477\": \"Value for 477\",\n\"478\": \"Value for 478\",\n\"479\": \"Value for 479\",\n\"480\": \"Value for 480\",\n\"481\": \"Value for 481\",\n\"482\": \"Value for 482\",\n\"483\": \"Value for 483\",\n\"484\": \"Value for 484\",\n\"485\": \"Value for 485\",\n\"486\": \"Value for 486\",\n\"487\": \"Value for 487\",\n\"488\": \"Value for 488\",\n\"489\": \"Value for 489\",\n\"490\": \"Value for 490\",\n\"491\": \"Value for 491\",\n\"492\": \"Value for 492\",\n\"493\": \"Value for 493\",\n\"494\": \"Value for 494\",\n\"495\": \"Value for 495\",\n\"496\": \"Value for 496\",\n\"497\": \"Value for 497\",\n\"498\": \"Value for 498\",\n\"499\": \"Value for 499\",\n\"500\": \"Value for 500\"\n}}}");
        getNewHttpCallBean1.setPostQueryObjects(map2);
        // Get current time for getting log entries later

        Timestamp getTimeAsTimeStamp7 = new Timestamp(System.currentTimeMillis());
        // Make JSON call to the operation requesting an XML response
        cougarManager1.makeRestCougarHTTPCall(getNewHttpCallBean1, com.betfair.testing.utils.cougar.enums.CougarMessageProtocolRequestTypeEnum.RESTJSON, com.betfair.testing.utils.cougar.enums.CougarMessageContentTypeEnum.XML);
        // Make JSON call to the operation requesting a JSON response
        cougarManager1.makeRestCougarHTTPCall(getNewHttpCallBean1, com.betfair.testing.utils.cougar.enums.CougarMessageProtocolRequestTypeEnum.RESTJSON, com.betfair.testing.utils.cougar.enums.CougarMessageContentTypeEnum.JSON);
        // Create the expected response as an XML document
        XMLHelpers xMLHelpers4 = new XMLHelpers();
        Document createAsDocument10 = xMLHelpers4.getXMLObjectFromString("<SimpleMapOperationResponse><SimpleMapOperationResponseObject><responseMap><entry key=\"1\"><String>Value for 1</String></entry><entry key=\"10\"><String>Value for 10</String></entry><entry key=\"100\"><String>Value for 100</String></entry><entry key=\"101\"><String>Value for 101</String></entry><entry key=\"102\"><String>Value for 102</String></entry><entry key=\"103\"><String>Value for 103</String></entry><entry key=\"104\"><String>Value for 104</String></entry><entry key=\"105\"><String>Value for 105</String></entry><entry key=\"106\"><String>Value for 106</String></entry><entry key=\"107\"><String>Value for 107</String></entry><entry key=\"108\"><String>Value for 108</String></entry><entry key=\"109\"><String>Value for 109</String></entry><entry key=\"11\"><String>Value for 11</String></entry><entry key=\"110\"><String>Value for 110</String></entry><entry key=\"111\"><String>Value for 111</String></entry><entry key=\"112\"><String>Value for 112</String></entry><entry key=\"113\"><String>Value for 113</String></entry><entry key=\"114\"><String>Value for 114</String></entry><entry key=\"115\"><String>Value for 115</String></entry><entry key=\"116\"><String>Value for 116</String></entry><entry key=\"117\"><String>Value for 117</String></entry><entry key=\"118\"><String>Value for 118</String></entry><entry key=\"119\"><String>Value for 119</String></entry><entry key=\"12\"><String>Value for 12</String></entry><entry key=\"120\"><String>Value for 120</String></entry><entry key=\"121\"><String>Value for 121</String></entry><entry key=\"122\"><String>Value for 122</String></entry><entry key=\"123\"><String>Value for 123</String></entry><entry key=\"124\"><String>Value for 124</String></entry><entry key=\"125\"><String>Value for 125</String></entry><entry key=\"126\"><String>Value for 126</String></entry><entry key=\"127\"><String>Value for 127</String></entry><entry key=\"128\"><String>Value for 128</String></entry><entry key=\"129\"><String>Value for 129</String></entry><entry key=\"13\"><String>Value for 13</String></entry><entry key=\"130\"><String>Value for 130</String></entry><entry key=\"131\"><String>Value for 131</String></entry><entry key=\"132\"><String>Value for 132</String></entry><entry key=\"133\"><String>Value for 133</String></entry><entry key=\"134\"><String>Value for 134</String></entry><entry key=\"135\"><String>Value for 135</String></entry><entry key=\"136\"><String>Value for 136</String></entry><entry key=\"137\"><String>Value for 137</String></entry><entry key=\"138\"><String>Value for 138</String></entry><entry key=\"139\"><String>Value for 139</String></entry><entry key=\"14\"><String>Value for 14</String></entry><entry key=\"140\"><String>Value for 140</String></entry><entry key=\"141\"><String>Value for 141</String></entry><entry key=\"142\"><String>Value for 142</String></entry><entry key=\"143\"><String>Value for 143</String></entry><entry key=\"144\"><String>Value for 144</String></entry><entry key=\"145\"><String>Value for 145</String></entry><entry key=\"146\"><String>Value for 146</String></entry><entry key=\"147\"><String>Value for 147</String></entry><entry key=\"148\"><String>Value for 148</String></entry><entry key=\"149\"><String>Value for 149</String></entry><entry key=\"15\"><String>Value for 15</String></entry><entry key=\"150\"><String>Value for 150</String></entry><entry key=\"151\"><String>Value for 151</String></entry><entry key=\"152\"><String>Value for 152</String></entry><entry key=\"153\"><String>Value for 153</String></entry><entry key=\"154\"><String>Value for 154</String></entry><entry key=\"155\"><String>Value for 155</String></entry><entry key=\"156\"><String>Value for 156</String></entry><entry key=\"157\"><String>Value for 157</String></entry><entry key=\"158\"><String>Value for 158</String></entry><entry key=\"159\"><String>Value for 159</String></entry><entry key=\"16\"><String>Value for 16</String></entry><entry key=\"160\"><String>Value for 160</String></entry><entry key=\"161\"><String>Value for 161</String></entry><entry key=\"162\"><String>Value for 162</String></entry><entry key=\"163\"><String>Value for 163</String></entry><entry key=\"164\"><String>Value for 164</String></entry><entry key=\"165\"><String>Value for 165</String></entry><entry key=\"166\"><String>Value for 166</String></entry><entry key=\"167\"><String>Value for 167</String></entry><entry key=\"168\"><String>Value for 168</String></entry><entry key=\"169\"><String>Value for 169</String></entry><entry key=\"17\"><String>Value for 17</String></entry><entry key=\"170\"><String>Value for 170</String></entry><entry key=\"171\"><String>Value for 171</String></entry><entry key=\"172\"><String>Value for 172</String></entry><entry key=\"173\"><String>Value for 173</String></entry><entry key=\"174\"><String>Value for 174</String></entry><entry key=\"175\"><String>Value for 175</String></entry><entry key=\"176\"><String>Value for 176</String></entry><entry key=\"177\"><String>Value for 177</String></entry><entry key=\"178\"><String>Value for 178</String></entry><entry key=\"179\"><String>Value for 179</String></entry><entry key=\"18\"><String>Value for 18</String></entry><entry key=\"180\"><String>Value for 180</String></entry><entry key=\"181\"><String>Value for 181</String></entry><entry key=\"182\"><String>Value for 182</String></entry><entry key=\"183\"><String>Value for 183</String></entry><entry key=\"184\"><String>Value for 184</String></entry><entry key=\"185\"><String>Value for 185</String></entry><entry key=\"186\"><String>Value for 186</String></entry><entry key=\"187\"><String>Value for 187</String></entry><entry key=\"188\"><String>Value for 188</String></entry><entry key=\"189\"><String>Value for 189</String></entry><entry key=\"19\"><String>Value for 19</String></entry><entry key=\"190\"><String>Value for 190</String></entry><entry key=\"191\"><String>Value for 191</String></entry><entry key=\"192\"><String>Value for 192</String></entry><entry key=\"193\"><String>Value for 193</String></entry><entry key=\"194\"><String>Value for 194</String></entry><entry key=\"195\"><String>Value for 195</String></entry><entry key=\"196\"><String>Value for 196</String></entry><entry key=\"197\"><String>Value for 197</String></entry><entry key=\"198\"><String>Value for 198</String></entry><entry key=\"199\"><String>Value for 199</String></entry><entry key=\"2\"><String>Value for 2</String></entry><entry key=\"20\"><String>Value for 20</String></entry><entry key=\"200\"><String>Value for 200</String></entry><entry key=\"201\"><String>Value for 201</String></entry><entry key=\"202\"><String>Value for 202</String></entry><entry key=\"203\"><String>Value for 203</String></entry><entry key=\"204\"><String>Value for 204</String></entry><entry key=\"205\"><String>Value for 205</String></entry><entry key=\"206\"><String>Value for 206</String></entry><entry key=\"207\"><String>Value for 207</String></entry><entry key=\"208\"><String>Value for 208</String></entry><entry key=\"209\"><String>Value for 209</String></entry><entry key=\"21\"><String>Value for 21</String></entry><entry key=\"210\"><String>Value for 210</String></entry><entry key=\"211\"><String>Value for 211</String></entry><entry key=\"212\"><String>Value for 212</String></entry><entry key=\"213\"><String>Value for 213</String></entry><entry key=\"214\"><String>Value for 214</String></entry><entry key=\"215\"><String>Value for 215</String></entry><entry key=\"216\"><String>Value for 216</String></entry><entry key=\"217\"><String>Value for 217</String></entry><entry key=\"218\"><String>Value for 218</String></entry><entry key=\"219\"><String>Value for 219</String></entry><entry key=\"22\"><String>Value for 22</String></entry><entry key=\"220\"><String>Value for 220</String></entry><entry key=\"221\"><String>Value for 221</String></entry><entry key=\"222\"><String>Value for 222</String></entry><entry key=\"223\"><String>Value for 223</String></entry><entry key=\"224\"><String>Value for 224</String></entry><entry key=\"225\"><String>Value for 225</String></entry><entry key=\"226\"><String>Value for 226</String></entry><entry key=\"227\"><String>Value for 227</String></entry><entry key=\"228\"><String>Value for 228</String></entry><entry key=\"229\"><String>Value for 229</String></entry><entry key=\"23\"><String>Value for 23</String></entry><entry key=\"230\"><String>Value for 230</String></entry><entry key=\"231\"><String>Value for 231</String></entry><entry key=\"232\"><String>Value for 232</String></entry><entry key=\"233\"><String>Value for 233</String></entry><entry key=\"234\"><String>Value for 234</String></entry><entry key=\"235\"><String>Value for 235</String></entry><entry key=\"236\"><String>Value for 236</String></entry><entry key=\"237\"><String>Value for 237</String></entry><entry key=\"238\"><String>Value for 238</String></entry><entry key=\"239\"><String>Value for 239</String></entry><entry key=\"24\"><String>Value for 24</String></entry><entry key=\"240\"><String>Value for 240</String></entry><entry key=\"241\"><String>Value for 241</String></entry><entry key=\"242\"><String>Value for 242</String></entry><entry key=\"243\"><String>Value for 243</String></entry><entry key=\"244\"><String>Value for 244</String></entry><entry key=\"245\"><String>Value for 245</String></entry><entry key=\"246\"><String>Value for 246</String></entry><entry key=\"247\"><String>Value for 247</String></entry><entry key=\"248\"><String>Value for 248</String></entry><entry key=\"249\"><String>Value for 249</String></entry><entry key=\"25\"><String>Value for 25</String></entry><entry key=\"250\"><String>Value for 250</String></entry><entry key=\"251\"><String>Value for 251</String></entry><entry key=\"252\"><String>Value for 252</String></entry><entry key=\"253\"><String>Value for 253</String></entry><entry key=\"254\"><String>Value for 254</String></entry><entry key=\"255\"><String>Value for 255</String></entry><entry key=\"256\"><String>Value for 256</String></entry><entry key=\"257\"><String>Value for 257</String></entry><entry key=\"258\"><String>Value for 258</String></entry><entry key=\"259\"><String>Value for 259</String></entry><entry key=\"26\"><String>Value for 26</String></entry><entry key=\"260\"><String>Value for 260</String></entry><entry key=\"261\"><String>Value for 261</String></entry><entry key=\"262\"><String>Value for 262</String></entry><entry key=\"263\"><String>Value for 263</String></entry><entry key=\"264\"><String>Value for 264</String></entry><entry key=\"265\"><String>Value for 265</String></entry><entry key=\"266\"><String>Value for 266</String></entry><entry key=\"267\"><String>Value for 267</String></entry><entry key=\"268\"><String>Value for 268</String></entry><entry key=\"269\"><String>Value for 269</String></entry><entry key=\"27\"><String>Value for 27</String></entry><entry key=\"270\"><String>Value for 270</String></entry><entry key=\"271\"><String>Value for 271</String></entry><entry key=\"272\"><String>Value for 272</String></entry><entry key=\"273\"><String>Value for 273</String></entry><entry key=\"274\"><String>Value for 274</String></entry><entry key=\"275\"><String>Value for 275</String></entry><entry key=\"276\"><String>Value for 276</String></entry><entry key=\"277\"><String>Value for 277</String></entry><entry key=\"278\"><String>Value for 278</String></entry><entry key=\"279\"><String>Value for 279</String></entry><entry key=\"28\"><String>Value for 28</String></entry><entry key=\"280\"><String>Value for 280</String></entry><entry key=\"281\"><String>Value for 281</String></entry><entry key=\"282\"><String>Value for 282</String></entry><entry key=\"283\"><String>Value for 283</String></entry><entry key=\"284\"><String>Value for 284</String></entry><entry key=\"285\"><String>Value for 285</String></entry><entry key=\"286\"><String>Value for 286</String></entry><entry key=\"287\"><String>Value for 287</String></entry><entry key=\"288\"><String>Value for 288</String></entry><entry key=\"289\"><String>Value for 289</String></entry><entry key=\"29\"><String>Value for 29</String></entry><entry key=\"290\"><String>Value for 290</String></entry><entry key=\"291\"><String>Value for 291</String></entry><entry key=\"292\"><String>Value for 292</String></entry><entry key=\"293\"><String>Value for 293</String></entry><entry key=\"294\"><String>Value for 294</String></entry><entry key=\"295\"><String>Value for 295</String></entry><entry key=\"296\"><String>Value for 296</String></entry><entry key=\"297\"><String>Value for 297</String></entry><entry key=\"298\"><String>Value for 298</String></entry><entry key=\"299\"><String>Value for 299</String></entry><entry key=\"3\"><String>Value for 3</String></entry><entry key=\"30\"><String>Value for 30</String></entry><entry key=\"300\"><String>Value for 300</String></entry><entry key=\"301\"><String>Value for 301</String></entry><entry key=\"302\"><String>Value for 302</String></entry><entry key=\"303\"><String>Value for 303</String></entry><entry key=\"304\"><String>Value for 304</String></entry><entry key=\"305\"><String>Value for 305</String></entry><entry key=\"306\"><String>Value for 306</String></entry><entry key=\"307\"><String>Value for 307</String></entry><entry key=\"308\"><String>Value for 308</String></entry><entry key=\"309\"><String>Value for 309</String></entry><entry key=\"31\"><String>Value for 31</String></entry><entry key=\"310\"><String>Value for 310</String></entry><entry key=\"311\"><String>Value for 311</String></entry><entry key=\"312\"><String>Value for 312</String></entry><entry key=\"313\"><String>Value for 313</String></entry><entry key=\"314\"><String>Value for 314</String></entry><entry key=\"315\"><String>Value for 315</String></entry><entry key=\"316\"><String>Value for 316</String></entry><entry key=\"317\"><String>Value for 317</String></entry><entry key=\"318\"><String>Value for 318</String></entry><entry key=\"319\"><String>Value for 319</String></entry><entry key=\"32\"><String>Value for 32</String></entry><entry key=\"320\"><String>Value for 320</String></entry><entry key=\"321\"><String>Value for 321</String></entry><entry key=\"322\"><String>Value for 322</String></entry><entry key=\"323\"><String>Value for 323</String></entry><entry key=\"324\"><String>Value for 324</String></entry><entry key=\"325\"><String>Value for 325</String></entry><entry key=\"326\"><String>Value for 326</String></entry><entry key=\"327\"><String>Value for 327</String></entry><entry key=\"328\"><String>Value for 328</String></entry><entry key=\"329\"><String>Value for 329</String></entry><entry key=\"33\"><String>Value for 33</String></entry><entry key=\"330\"><String>Value for 330</String></entry><entry key=\"331\"><String>Value for 331</String></entry><entry key=\"332\"><String>Value for 332</String></entry><entry key=\"333\"><String>Value for 333</String></entry><entry key=\"334\"><String>Value for 334</String></entry><entry key=\"335\"><String>Value for 335</String></entry><entry key=\"336\"><String>Value for 336</String></entry><entry key=\"337\"><String>Value for 337</String></entry><entry key=\"338\"><String>Value for 338</String></entry><entry key=\"339\"><String>Value for 339</String></entry><entry key=\"34\"><String>Value for 34</String></entry><entry key=\"340\"><String>Value for 340</String></entry><entry key=\"341\"><String>Value for 341</String></entry><entry key=\"342\"><String>Value for 342</String></entry><entry key=\"343\"><String>Value for 343</String></entry><entry key=\"344\"><String>Value for 344</String></entry><entry key=\"345\"><String>Value for 345</String></entry><entry key=\"346\"><String>Value for 346</String></entry><entry key=\"347\"><String>Value for 347</String></entry><entry key=\"348\"><String>Value for 348</String></entry><entry key=\"349\"><String>Value for 349</String></entry><entry key=\"35\"><String>Value for 35</String></entry><entry key=\"350\"><String>Value for 350</String></entry><entry key=\"351\"><String>Value for 351</String></entry><entry key=\"352\"><String>Value for 352</String></entry><entry key=\"353\"><String>Value for 353</String></entry><entry key=\"354\"><String>Value for 354</String></entry><entry key=\"355\"><String>Value for 355</String></entry><entry key=\"356\"><String>Value for 356</String></entry><entry key=\"357\"><String>Value for 357</String></entry><entry key=\"358\"><String>Value for 358</String></entry><entry key=\"359\"><String>Value for 359</String></entry><entry key=\"36\"><String>Value for 36</String></entry><entry key=\"360\"><String>Value for 360</String></entry><entry key=\"361\"><String>Value for 361</String></entry><entry key=\"362\"><String>Value for 362</String></entry><entry key=\"363\"><String>Value for 363</String></entry><entry key=\"364\"><String>Value for 364</String></entry><entry key=\"365\"><String>Value for 365</String></entry><entry key=\"366\"><String>Value for 366</String></entry><entry key=\"367\"><String>Value for 367</String></entry><entry key=\"368\"><String>Value for 368</String></entry><entry key=\"369\"><String>Value for 369</String></entry><entry key=\"37\"><String>Value for 37</String></entry><entry key=\"370\"><String>Value for 370</String></entry><entry key=\"371\"><String>Value for 371</String></entry><entry key=\"372\"><String>Value for 372</String></entry><entry key=\"373\"><String>Value for 373</String></entry><entry key=\"374\"><String>Value for 374</String></entry><entry key=\"375\"><String>Value for 375</String></entry><entry key=\"376\"><String>Value for 376</String></entry><entry key=\"377\"><String>Value for 377</String></entry><entry key=\"378\"><String>Value for 378</String></entry><entry key=\"379\"><String>Value for 379</String></entry><entry key=\"38\"><String>Value for 38</String></entry><entry key=\"380\"><String>Value for 380</String></entry><entry key=\"381\"><String>Value for 381</String></entry><entry key=\"382\"><String>Value for 382</String></entry><entry key=\"383\"><String>Value for 383</String></entry><entry key=\"384\"><String>Value for 384</String></entry><entry key=\"385\"><String>Value for 385</String></entry><entry key=\"386\"><String>Value for 386</String></entry><entry key=\"387\"><String>Value for 387</String></entry><entry key=\"388\"><String>Value for 388</String></entry><entry key=\"389\"><String>Value for 389</String></entry><entry key=\"39\"><String>Value for 39</String></entry><entry key=\"390\"><String>Value for 390</String></entry><entry key=\"391\"><String>Value for 391</String></entry><entry key=\"392\"><String>Value for 392</String></entry><entry key=\"393\"><String>Value for 393</String></entry><entry key=\"394\"><String>Value for 394</String></entry><entry key=\"395\"><String>Value for 395</String></entry><entry key=\"396\"><String>Value for 396</String></entry><entry key=\"397\"><String>Value for 397</String></entry><entry key=\"398\"><String>Value for 398</String></entry><entry key=\"399\"><String>Value for 399</String></entry><entry key=\"4\"><String>Value for 4</String></entry><entry key=\"40\"><String>Value for 40</String></entry><entry key=\"400\"><String>Value for 400</String></entry><entry key=\"401\"><String>Value for 401</String></entry><entry key=\"402\"><String>Value for 402</String></entry><entry key=\"403\"><String>Value for 403</String></entry><entry key=\"404\"><String>Value for 404</String></entry><entry key=\"405\"><String>Value for 405</String></entry><entry key=\"406\"><String>Value for 406</String></entry><entry key=\"407\"><String>Value for 407</String></entry><entry key=\"408\"><String>Value for 408</String></entry><entry key=\"409\"><String>Value for 409</String></entry><entry key=\"41\"><String>Value for 41</String></entry><entry key=\"410\"><String>Value for 410</String></entry><entry key=\"411\"><String>Value for 411</String></entry><entry key=\"412\"><String>Value for 412</String></entry><entry key=\"413\"><String>Value for 413</String></entry><entry key=\"414\"><String>Value for 414</String></entry><entry key=\"415\"><String>Value for 415</String></entry><entry key=\"416\"><String>Value for 416</String></entry><entry key=\"417\"><String>Value for 417</String></entry><entry key=\"418\"><String>Value for 418</String></entry><entry key=\"419\"><String>Value for 419</String></entry><entry key=\"42\"><String>Value for 42</String></entry><entry key=\"420\"><String>Value for 420</String></entry><entry key=\"421\"><String>Value for 421</String></entry><entry key=\"422\"><String>Value for 422</String></entry><entry key=\"423\"><String>Value for 423</String></entry><entry key=\"424\"><String>Value for 424</String></entry><entry key=\"425\"><String>Value for 425</String></entry><entry key=\"426\"><String>Value for 426</String></entry><entry key=\"427\"><String>Value for 427</String></entry><entry key=\"428\"><String>Value for 428</String></entry><entry key=\"429\"><String>Value for 429</String></entry><entry key=\"43\"><String>Value for 43</String></entry><entry key=\"430\"><String>Value for 430</String></entry><entry key=\"431\"><String>Value for 431</String></entry><entry key=\"432\"><String>Value for 432</String></entry><entry key=\"433\"><String>Value for 433</String></entry><entry key=\"434\"><String>Value for 434</String></entry><entry key=\"435\"><String>Value for 435</String></entry><entry key=\"436\"><String>Value for 436</String></entry><entry key=\"437\"><String>Value for 437</String></entry><entry key=\"438\"><String>Value for 438</String></entry><entry key=\"439\"><String>Value for 439</String></entry><entry key=\"44\"><String>Value for 44</String></entry><entry key=\"440\"><String>Value for 440</String></entry><entry key=\"441\"><String>Value for 441</String></entry><entry key=\"442\"><String>Value for 442</String></entry><entry key=\"443\"><String>Value for 443</String></entry><entry key=\"444\"><String>Value for 444</String></entry><entry key=\"445\"><String>Value for 445</String></entry><entry key=\"446\"><String>Value for 446</String></entry><entry key=\"447\"><String>Value for 447</String></entry><entry key=\"448\"><String>Value for 448</String></entry><entry key=\"449\"><String>Value for 449</String></entry><entry key=\"45\"><String>Value for 45</String></entry><entry key=\"450\"><String>Value for 450</String></entry><entry key=\"451\"><String>Value for 451</String></entry><entry key=\"452\"><String>Value for 452</String></entry><entry key=\"453\"><String>Value for 453</String></entry><entry key=\"454\"><String>Value for 454</String></entry><entry key=\"455\"><String>Value for 455</String></entry><entry key=\"456\"><String>Value for 456</String></entry><entry key=\"457\"><String>Value for 457</String></entry><entry key=\"458\"><String>Value for 458</String></entry><entry key=\"459\"><String>Value for 459</String></entry><entry key=\"46\"><String>Value for 46</String></entry><entry key=\"460\"><String>Value for 460</String></entry><entry key=\"461\"><String>Value for 461</String></entry><entry key=\"462\"><String>Value for 462</String></entry><entry key=\"463\"><String>Value for 463</String></entry><entry key=\"464\"><String>Value for 464</String></entry><entry key=\"465\"><String>Value for 465</String></entry><entry key=\"466\"><String>Value for 466</String></entry><entry key=\"467\"><String>Value for 467</String></entry><entry key=\"468\"><String>Value for 468</String></entry><entry key=\"469\"><String>Value for 469</String></entry><entry key=\"47\"><String>Value for 47</String></entry><entry key=\"470\"><String>Value for 470</String></entry><entry key=\"471\"><String>Value for 471</String></entry><entry key=\"472\"><String>Value for 472</String></entry><entry key=\"473\"><String>Value for 473</String></entry><entry key=\"474\"><String>Value for 474</String></entry><entry key=\"475\"><String>Value for 475</String></entry><entry key=\"476\"><String>Value for 476</String></entry><entry key=\"477\"><String>Value for 477</String></entry><entry key=\"478\"><String>Value for 478</String></entry><entry key=\"479\"><String>Value for 479</String></entry><entry key=\"48\"><String>Value for 48</String></entry><entry key=\"480\"><String>Value for 480</String></entry><entry key=\"481\"><String>Value for 481</String></entry><entry key=\"482\"><String>Value for 482</String></entry><entry key=\"483\"><String>Value for 483</String></entry><entry key=\"484\"><String>Value for 484</String></entry><entry key=\"485\"><String>Value for 485</String></entry><entry key=\"486\"><String>Value for 486</String></entry><entry key=\"487\"><String>Value for 487</String></entry><entry key=\"488\"><String>Value for 488</String></entry><entry key=\"489\"><String>Value for 489</String></entry><entry key=\"49\"><String>Value for 49</String></entry><entry key=\"490\"><String>Value for 490</String></entry><entry key=\"491\"><String>Value for 491</String></entry><entry key=\"492\"><String>Value for 492</String></entry><entry key=\"493\"><String>Value for 493</String></entry><entry key=\"494\"><String>Value for 494</String></entry><entry key=\"495\"><String>Value for 495</String></entry><entry key=\"496\"><String>Value for 496</String></entry><entry key=\"497\"><String>Value for 497</String></entry><entry key=\"498\"><String>Value for 498</String></entry><entry key=\"499\"><String>Value for 499</String></entry><entry key=\"5\"><String>Value for 5</String></entry><entry key=\"50\"><String>Value for 50</String></entry><entry key=\"500\"><String>Value for 500</String></entry><entry key=\"51\"><String>Value for 51</String></entry><entry key=\"52\"><String>Value for 52</String></entry><entry key=\"53\"><String>Value for 53</String></entry><entry key=\"54\"><String>Value for 54</String></entry><entry key=\"55\"><String>Value for 55</String></entry><entry key=\"56\"><String>Value for 56</String></entry><entry key=\"57\"><String>Value for 57</String></entry><entry key=\"58\"><String>Value for 58</String></entry><entry key=\"59\"><String>Value for 59</String></entry><entry key=\"6\"><String>Value for 6</String></entry><entry key=\"60\"><String>Value for 60</String></entry><entry key=\"61\"><String>Value for 61</String></entry><entry key=\"62\"><String>Value for 62</String></entry><entry key=\"63\"><String>Value for 63</String></entry><entry key=\"64\"><String>Value for 64</String></entry><entry key=\"65\"><String>Value for 65</String></entry><entry key=\"66\"><String>Value for 66</String></entry><entry key=\"67\"><String>Value for 67</String></entry><entry key=\"68\"><String>Value for 68</String></entry><entry key=\"69\"><String>Value for 69</String></entry><entry key=\"7\"><String>Value for 7</String></entry><entry key=\"70\"><String>Value for 70</String></entry><entry key=\"71\"><String>Value for 71</String></entry><entry key=\"72\"><String>Value for 72</String></entry><entry key=\"73\"><String>Value for 73</String></entry><entry key=\"74\"><String>Value for 74</String></entry><entry key=\"75\"><String>Value for 75</String></entry><entry key=\"76\"><String>Value for 76</String></entry><entry key=\"77\"><String>Value for 77</String></entry><entry key=\"78\"><String>Value for 78</String></entry><entry key=\"79\"><String>Value for 79</String></entry><entry key=\"8\"><String>Value for 8</String></entry><entry key=\"80\"><String>Value for 80</String></entry><entry key=\"81\"><String>Value for 81</String></entry><entry key=\"82\"><String>Value for 82</String></entry><entry key=\"83\"><String>Value for 83</String></entry><entry key=\"84\"><String>Value for 84</String></entry><entry key=\"85\"><String>Value for 85</String></entry><entry key=\"86\"><String>Value for 86</String></entry><entry key=\"87\"><String>Value for 87</String></entry><entry key=\"88\"><String>Value for 88</String></entry><entry key=\"89\"><String>Value for 89</String></entry><entry key=\"9\"><String>Value for 9</String></entry><entry key=\"90\"><String>Value for 90</String></entry><entry key=\"91\"><String>Value for 91</String></entry><entry key=\"92\"><String>Value for 92</String></entry><entry key=\"93\"><String>Value for 93</String></entry><entry key=\"94\"><String>Value for 94</String></entry><entry key=\"95\"><String>Value for 95</String></entry><entry key=\"96\"><String>Value for 96</String></entry><entry key=\"97\"><String>Value for 97</String></entry><entry key=\"98\"><String>Value for 98</String></entry><entry key=\"99\"><String>Value for 99</String></entry></responseMap></SimpleMapOperationResponseObject></SimpleMapOperationResponse>");
        // Create the expected response as a JSON object
        JSONHelpers jSONHelpers5 = new JSONHelpers();
        JSONObject createAsJSONObject11 = jSONHelpers5.createAsJSONObject(new JSONObject("{\"responseMap\": {\"1\": \"Value for 1\",\"2\": \"Value for 2\",\"3\": \"Value for 3\",\"4\": \"Value for 4\",\"5\": \"Value for 5\",\"6\": \"Value for 6\",\"7\": \"Value for 7\",\"8\": \"Value for 8\",\"9\": \"Value for 9\",\"10\": \"Value for 10\",\"11\": \"Value for 11\",\"12\": \"Value for 12\",\"13\": \"Value for 13\",\"14\": \"Value for 14\",\"15\": \"Value for 15\",\"16\": \"Value for 16\",\"17\": \"Value for 17\",\"18\": \"Value for 18\",\"19\": \"Value for 19\",\"20\": \"Value for 20\",\"21\": \"Value for 21\",\"22\": \"Value for 22\",\"23\": \"Value for 23\",\"24\": \"Value for 24\",\"25\": \"Value for 25\",\"26\": \"Value for 26\",\"27\": \"Value for 27\",\"28\": \"Value for 28\",\"29\": \"Value for 29\",\"30\": \"Value for 30\",\"31\": \"Value for 31\",\"32\": \"Value for 32\",\"33\": \"Value for 33\",\"34\": \"Value for 34\",\"35\": \"Value for 35\",\"36\": \"Value for 36\",\"37\": \"Value for 37\",\"38\": \"Value for 38\",\"39\": \"Value for 39\",\"40\": \"Value for 40\",\"41\": \"Value for 41\",\"42\": \"Value for 42\",\"43\": \"Value for 43\",\"44\": \"Value for 44\",\"45\": \"Value for 45\",\"46\": \"Value for 46\",\"47\": \"Value for 47\",\"48\": \"Value for 48\",\"49\": \"Value for 49\",\"50\": \"Value for 50\",\"51\": \"Value for 51\",\"52\": \"Value for 52\",\"53\": \"Value for 53\",\"54\": \"Value for 54\",\"55\": \"Value for 55\",\"56\": \"Value for 56\",\"57\": \"Value for 57\",\"58\": \"Value for 58\",\"59\": \"Value for 59\",\"60\": \"Value for 60\",\"61\": \"Value for 61\",\"62\": \"Value for 62\",\"63\": \"Value for 63\",\"64\": \"Value for 64\",\"65\": \"Value for 65\",\"66\": \"Value for 66\",\"67\": \"Value for 67\",\"68\": \"Value for 68\",\"69\": \"Value for 69\",\"70\": \"Value for 70\",\"71\": \"Value for 71\",\"72\": \"Value for 72\",\"73\": \"Value for 73\",\"74\": \"Value for 74\",\"75\": \"Value for 75\",\"76\": \"Value for 76\",\"77\": \"Value for 77\",\"78\": \"Value for 78\",\"79\": \"Value for 79\",\"80\": \"Value for 80\",\"81\": \"Value for 81\",\"82\": \"Value for 82\",\"83\": \"Value for 83\",\"84\": \"Value for 84\",\"85\": \"Value for 85\",\"86\": \"Value for 86\",\"87\": \"Value for 87\",\"88\": \"Value for 88\",\"89\": \"Value for 89\",\"90\": \"Value for 90\",\"91\": \"Value for 91\",\"92\": \"Value for 92\",\"93\": \"Value for 93\",\"94\": \"Value for 94\",\"95\": \"Value for 95\",\"96\": \"Value for 96\",\"97\": \"Value for 97\",\"98\": \"Value for 98\",\"99\": \"Value for 99\",\"100\": \"Value for 100\",\"101\": \"Value for 101\",\"102\": \"Value for 102\",\"103\": \"Value for 103\",\"104\": \"Value for 104\",\"105\": \"Value for 105\",\"106\": \"Value for 106\",\"107\": \"Value for 107\",\"108\": \"Value for 108\",\"109\": \"Value for 109\",\"110\": \"Value for 110\",\"111\": \"Value for 111\",\"112\": \"Value for 112\",\"113\": \"Value for 113\",\"114\": \"Value for 114\",\"115\": \"Value for 115\",\"116\": \"Value for 116\",\"117\": \"Value for 117\",\"118\": \"Value for 118\",\"119\": \"Value for 119\",\"120\": \"Value for 120\",\"121\": \"Value for 121\",\"122\": \"Value for 122\",\"123\": \"Value for 123\",\"124\": \"Value for 124\",\"125\": \"Value for 125\",\"126\": \"Value for 126\",\"127\": \"Value for 127\",\"128\": \"Value for 128\",\"129\": \"Value for 129\",\"130\": \"Value for 130\",\"131\": \"Value for 131\",\"132\": \"Value for 132\",\"133\": \"Value for 133\",\"134\": \"Value for 134\",\"135\": \"Value for 135\",\"136\": \"Value for 136\",\"137\": \"Value for 137\",\"138\": \"Value for 138\",\"139\": \"Value for 139\",\"140\": \"Value for 140\",\"141\": \"Value for 141\",\"142\": \"Value for 142\",\"143\": \"Value for 143\",\"144\": \"Value for 144\",\"145\": \"Value for 145\",\"146\": \"Value for 146\",\"147\": \"Value for 147\",\"148\": \"Value for 148\",\"149\": \"Value for 149\",\"150\": \"Value for 150\",\"151\": \"Value for 151\",\"152\": \"Value for 152\",\"153\": \"Value for 153\",\"154\": \"Value for 154\",\"155\": \"Value for 155\",\"156\": \"Value for 156\",\"157\": \"Value for 157\",\"158\": \"Value for 158\",\"159\": \"Value for 159\",\"160\": \"Value for 160\",\"161\": \"Value for 161\",\"162\": \"Value for 162\",\"163\": \"Value for 163\",\"164\": \"Value for 164\",\"165\": \"Value for 165\",\"166\": \"Value for 166\",\"167\": \"Value for 167\",\"168\": \"Value for 168\",\"169\": \"Value for 169\",\"170\": \"Value for 170\",\"171\": \"Value for 171\",\"172\": \"Value for 172\",\"173\": \"Value for 173\",\"174\": \"Value for 174\",\"175\": \"Value for 175\",\"176\": \"Value for 176\",\"177\": \"Value for 177\",\"178\": \"Value for 178\",\"179\": \"Value for 179\",\"180\": \"Value for 180\",\"181\": \"Value for 181\",\"182\": \"Value for 182\",\"183\": \"Value for 183\",\"184\": \"Value for 184\",\"185\": \"Value for 185\",\"186\": \"Value for 186\",\"187\": \"Value for 187\",\"188\": \"Value for 188\",\"189\": \"Value for 189\",\"190\": \"Value for 190\",\"191\": \"Value for 191\",\"192\": \"Value for 192\",\"193\": \"Value for 193\",\"194\": \"Value for 194\",\"195\": \"Value for 195\",\"196\": \"Value for 196\",\"197\": \"Value for 197\",\"198\": \"Value for 198\",\"199\": \"Value for 199\",\"200\": \"Value for 200\",\"201\": \"Value for 201\",\"202\": \"Value for 202\",\"203\": \"Value for 203\",\"204\": \"Value for 204\",\"205\": \"Value for 205\",\"206\": \"Value for 206\",\"207\": \"Value for 207\",\"208\": \"Value for 208\",\"209\": \"Value for 209\",\"210\": \"Value for 210\",\"211\": \"Value for 211\",\"212\": \"Value for 212\",\"213\": \"Value for 213\",\"214\": \"Value for 214\",\"215\": \"Value for 215\",\"216\": \"Value for 216\",\"217\": \"Value for 217\",\"218\": \"Value for 218\",\"219\": \"Value for 219\",\"220\": \"Value for 220\",\"221\": \"Value for 221\",\"222\": \"Value for 222\",\"223\": \"Value for 223\",\"224\": \"Value for 224\",\"225\": \"Value for 225\",\"226\": \"Value for 226\",\"227\": \"Value for 227\",\"228\": \"Value for 228\",\"229\": \"Value for 229\",\"230\": \"Value for 230\",\"231\": \"Value for 231\",\"232\": \"Value for 232\",\"233\": \"Value for 233\",\"234\": \"Value for 234\",\"235\": \"Value for 235\",\"236\": \"Value for 236\",\"237\": \"Value for 237\",\"238\": \"Value for 238\",\"239\": \"Value for 239\",\"240\": \"Value for 240\",\"241\": \"Value for 241\",\"242\": \"Value for 242\",\"243\": \"Value for 243\",\"244\": \"Value for 244\",\"245\": \"Value for 245\",\"246\": \"Value for 246\",\"247\": \"Value for 247\",\"248\": \"Value for 248\",\"249\": \"Value for 249\",\"250\": \"Value for 250\",\"251\": \"Value for 251\",\"252\": \"Value for 252\",\"253\": \"Value for 253\",\"254\": \"Value for 254\",\"255\": \"Value for 255\",\"256\": \"Value for 256\",\"257\": \"Value for 257\",\"258\": \"Value for 258\",\"259\": \"Value for 259\",\"260\": \"Value for 260\",\"261\": \"Value for 261\",\"262\": \"Value for 262\",\"263\": \"Value for 263\",\"264\": \"Value for 264\",\"265\": \"Value for 265\",\"266\": \"Value for 266\",\"267\": \"Value for 267\",\"268\": \"Value for 268\",\"269\": \"Value for 269\",\"270\": \"Value for 270\",\"271\": \"Value for 271\",\"272\": \"Value for 272\",\"273\": \"Value for 273\",\"274\": \"Value for 274\",\"275\": \"Value for 275\",\"276\": \"Value for 276\",\"277\": \"Value for 277\",\"278\": \"Value for 278\",\"279\": \"Value for 279\",\"280\": \"Value for 280\",\"281\": \"Value for 281\",\"282\": \"Value for 282\",\"283\": \"Value for 283\",\"284\": \"Value for 284\",\"285\": \"Value for 285\",\"286\": \"Value for 286\",\"287\": \"Value for 287\",\"288\": \"Value for 288\",\"289\": \"Value for 289\",\"290\": \"Value for 290\",\"291\": \"Value for 291\",\"292\": \"Value for 292\",\"293\": \"Value for 293\",\"294\": \"Value for 294\",\"295\": \"Value for 295\",\"296\": \"Value for 296\",\"297\": \"Value for 297\",\"298\": \"Value for 298\",\"299\": \"Value for 299\",\"300\": \"Value for 300\",\"301\": \"Value for 301\",\"302\": \"Value for 302\",\"303\": \"Value for 303\",\"304\": \"Value for 304\",\"305\": \"Value for 305\",\"306\": \"Value for 306\",\"307\": \"Value for 307\",\"308\": \"Value for 308\",\"309\": \"Value for 309\",\"310\": \"Value for 310\",\"311\": \"Value for 311\",\"312\": \"Value for 312\",\"313\": \"Value for 313\",\"314\": \"Value for 314\",\"315\": \"Value for 315\",\"316\": \"Value for 316\",\"317\": \"Value for 317\",\"318\": \"Value for 318\",\"319\": \"Value for 319\",\"320\": \"Value for 320\",\"321\": \"Value for 321\",\"322\": \"Value for 322\",\"323\": \"Value for 323\",\"324\": \"Value for 324\",\"325\": \"Value for 325\",\"326\": \"Value for 326\",\"327\": \"Value for 327\",\"328\": \"Value for 328\",\"329\": \"Value for 329\",\"330\": \"Value for 330\",\"331\": \"Value for 331\",\"332\": \"Value for 332\",\"333\": \"Value for 333\",\"334\": \"Value for 334\",\"335\": \"Value for 335\",\"336\": \"Value for 336\",\"337\": \"Value for 337\",\"338\": \"Value for 338\",\"339\": \"Value for 339\",\"340\": \"Value for 340\",\"341\": \"Value for 341\",\"342\": \"Value for 342\",\"343\": \"Value for 343\",\"344\": \"Value for 344\",\"345\": \"Value for 345\",\"346\": \"Value for 346\",\"347\": \"Value for 347\",\"348\": \"Value for 348\",\"349\": \"Value for 349\",\"350\": \"Value for 350\",\"351\": \"Value for 351\",\"352\": \"Value for 352\",\"353\": \"Value for 353\",\"354\": \"Value for 354\",\"355\": \"Value for 355\",\"356\": \"Value for 356\",\"357\": \"Value for 357\",\"358\": \"Value for 358\",\"359\": \"Value for 359\",\"360\": \"Value for 360\",\"361\": \"Value for 361\",\"362\": \"Value for 362\",\"363\": \"Value for 363\",\"364\": \"Value for 364\",\"365\": \"Value for 365\",\"366\": \"Value for 366\",\"367\": \"Value for 367\",\"368\": \"Value for 368\",\"369\": \"Value for 369\",\"370\": \"Value for 370\",\"371\": \"Value for 371\",\"372\": \"Value for 372\",\"373\": \"Value for 373\",\"374\": \"Value for 374\",\"375\": \"Value for 375\",\"376\": \"Value for 376\",\"377\": \"Value for 377\",\"378\": \"Value for 378\",\"379\": \"Value for 379\",\"380\": \"Value for 380\",\"381\": \"Value for 381\",\"382\": \"Value for 382\",\"383\": \"Value for 383\",\"384\": \"Value for 384\",\"385\": \"Value for 385\",\"386\": \"Value for 386\",\"387\": \"Value for 387\",\"388\": \"Value for 388\",\"389\": \"Value for 389\",\"390\": \"Value for 390\",\"391\": \"Value for 391\",\"392\": \"Value for 392\",\"393\": \"Value for 393\",\"394\": \"Value for 394\",\"395\": \"Value for 395\",\"396\": \"Value for 396\",\"397\": \"Value for 397\",\"398\": \"Value for 398\",\"399\": \"Value for 399\",\"400\": \"Value for 400\",\"401\": \"Value for 401\",\"402\": \"Value for 402\",\"403\": \"Value for 403\",\"404\": \"Value for 404\",\"405\": \"Value for 405\",\"406\": \"Value for 406\",\"407\": \"Value for 407\",\"408\": \"Value for 408\",\"409\": \"Value for 409\",\"410\": \"Value for 410\",\"411\": \"Value for 411\",\"412\": \"Value for 412\",\"413\": \"Value for 413\",\"414\": \"Value for 414\",\"415\": \"Value for 415\",\"416\": \"Value for 416\",\"417\": \"Value for 417\",\"418\": \"Value for 418\",\"419\": \"Value for 419\",\"420\": \"Value for 420\",\"421\": \"Value for 421\",\"422\": \"Value for 422\",\"423\": \"Value for 423\",\"424\": \"Value for 424\",\"425\": \"Value for 425\",\"426\": \"Value for 426\",\"427\": \"Value for 427\",\"428\": \"Value for 428\",\"429\": \"Value for 429\",\"430\": \"Value for 430\",\"431\": \"Value for 431\",\"432\": \"Value for 432\",\"433\": \"Value for 433\",\"434\": \"Value for 434\",\"435\": \"Value for 435\",\"436\": \"Value for 436\",\"437\": \"Value for 437\",\"438\": \"Value for 438\",\"439\": \"Value for 439\",\"440\": \"Value for 440\",\"441\": \"Value for 441\",\"442\": \"Value for 442\",\"443\": \"Value for 443\",\"444\": \"Value for 444\",\"445\": \"Value for 445\",\"446\": \"Value for 446\",\"447\": \"Value for 447\",\"448\": \"Value for 448\",\"449\": \"Value for 449\",\"450\": \"Value for 450\",\"451\": \"Value for 451\",\"452\": \"Value for 452\",\"453\": \"Value for 453\",\"454\": \"Value for 454\",\"455\": \"Value for 455\",\"456\": \"Value for 456\",\"457\": \"Value for 457\",\"458\": \"Value for 458\",\"459\": \"Value for 459\",\"460\": \"Value for 460\",\"461\": \"Value for 461\",\"462\": \"Value for 462\",\"463\": \"Value for 463\",\"464\": \"Value for 464\",\"465\": \"Value for 465\",\"466\": \"Value for 466\",\"467\": \"Value for 467\",\"468\": \"Value for 468\",\"469\": \"Value for 469\",\"470\": \"Value for 470\",\"471\": \"Value for 471\",\"472\": \"Value for 472\",\"473\": \"Value for 473\",\"474\": \"Value for 474\",\"475\": \"Value for 475\",\"476\": \"Value for 476\",\"477\": \"Value for 477\",\"478\": \"Value for 478\",\"479\": \"Value for 479\",\"480\": \"Value for 480\",\"481\": \"Value for 481\",\"482\": \"Value for 482\",\"483\": \"Value for 483\",\"484\": \"Value for 484\",\"485\": \"Value for 485\",\"486\": \"Value for 486\",\"487\": \"Value for 487\",\"488\": \"Value for 488\",\"489\": \"Value for 489\",\"490\": \"Value for 490\",\"491\": \"Value for 491\",\"492\": \"Value for 492\",\"493\": \"Value for 493\",\"494\": \"Value for 494\",\"495\": \"Value for 495\",\"496\": \"Value for 496\",\"497\": \"Value for 497\",\"498\": \"Value for 498\",\"499\": \"Value for 499\",\"500\": \"Value for 500\"}}"));
        // Check the 2 responses are as expected
        HttpResponseBean response6 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTJSONXML);
        AssertionUtils.multiAssertEquals(createAsDocument10, response6.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response6.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response6.getHttpStatusText());
        
        HttpResponseBean response7 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTJSONJSON);
        AssertionUtils.multiAssertEquals(createAsJSONObject11, response7.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response7.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response7.getHttpStatusText());
        
        // generalHelpers.pauseTest(500L);
        // Check the log entries are as expected
        
        cougarManager1.verifyRequestLogEntriesAfterDate(getTimeAsTimeStamp7, new RequestLogRequirement("2.8", "simpleMapOperation"),new RequestLogRequirement("2.8", "simpleMapOperation") );
    }

}
