/*
 * Copyright 2013, The Sporting Exchange Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// Originally from UpdatedComponentTests/StandardTesting/REST/Rest_Post_RequestTypes_Set_SimpleSet_Volume.xls;
package com.betfair.cougar.tests.updatedcomponenttests.standardtesting.rest;

import com.betfair.testing.utils.cougar.misc.XMLHelpers;
import com.betfair.testing.utils.cougar.assertions.AssertionUtils;
import com.betfair.testing.utils.cougar.beans.HttpCallBean;
import com.betfair.testing.utils.cougar.beans.HttpResponseBean;
import com.betfair.testing.utils.cougar.enums.CougarMessageProtocolRequestTypeEnum;
import com.betfair.testing.utils.cougar.manager.CougarManager;
import com.betfair.testing.utils.cougar.manager.RequestLogRequirement;

import org.testng.annotations.Test;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilderFactory;
import java.io.ByteArrayInputStream;
import java.sql.Timestamp;
import java.util.Map;

/**
 * Ensure that when a Rest (XML/JSON) Post operation is performed, Cougar can correctly handle a simple Set with a large volume of entries in the post body
 */
public class RestPostRequestTypesSetSimpleSetVolumeTest {
    @Test
    public void doTest() throws Exception {
        // Set up the Http Call Bean to make the request
        CougarManager cougarManager1 = CougarManager.getInstance();
        HttpCallBean getNewHttpCallBean1 = cougarManager1.getNewHttpCallBean("87.248.113.14");
        cougarManager1 = cougarManager1;
        
        getNewHttpCallBean1.setOperationName("simpleSetOperation");
        
        getNewHttpCallBean1.setServiceName("baseline", "cougarBaseline");
        
        getNewHttpCallBean1.setVersion("v2");
        // Set the post body to contain a simple set with a large volume of entries
        getNewHttpCallBean1.setRestPostQueryObjects(DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream("<message><simpleSet><String>string 001</String><String>string 002</String><String>string 003</String><String>string 004</String><String>string 005</String><String>string 006</String><String>string 007</String><String>string 008</String><String>string 009</String><String>string 010</String><String>string 011</String><String>string 012</String><String>string 013</String><String>string 014</String><String>string 015</String><String>string 016</String><String>string 017</String><String>string 018</String><String>string 019</String><String>string 020</String><String>string 021</String><String>string 022</String><String>string 023</String><String>string 024</String><String>string 025</String><String>string 026</String><String>string 027</String><String>string 028</String><String>string 029</String><String>string 030</String><String>string 031</String><String>string 032</String><String>string 033</String><String>string 034</String><String>string 035</String><String>string 036</String><String>string 037</String><String>string 038</String><String>string 039</String><String>string 040</String><String>string 041</String><String>string 042</String><String>string 043</String><String>string 044</String><String>string 045</String><String>string 046</String><String>string 047</String><String>string 048</String><String>string 049</String><String>string 050</String><String>string 051</String><String>string 052</String><String>string 053</String><String>string 054</String><String>string 055</String><String>string 056</String><String>string 057</String><String>string 058</String><String>string 059</String><String>string 060</String><String>string 061</String><String>string 062</String><String>string 063</String><String>string 064</String><String>string 065</String><String>string 066</String><String>string 067</String><String>string 068</String><String>string 069</String><String>string 070</String><String>string 071</String><String>string 072</String><String>string 073</String><String>string 074</String><String>string 075</String><String>string 076</String><String>string 077</String><String>string 078</String><String>string 079</String><String>string 080</String><String>string 081</String><String>string 082</String><String>string 083</String><String>string 084</String><String>string 085</String><String>string 086</String><String>string 087</String><String>string 088</String><String>string 089</String><String>string 090</String><String>string 091</String><String>string 092</String><String>string 093</String><String>string 094</String><String>string 095</String><String>string 096</String><String>string 097</String><String>string 098</String><String>string 099</String><String>string 100</String><String>string 101</String><String>string 102</String><String>string 103</String><String>string 104</String><String>string 105</String><String>string 106</String><String>string 107</String><String>string 108</String><String>string 109</String><String>string 110</String><String>string 111</String><String>string 112</String><String>string 113</String><String>string 114</String><String>string 115</String><String>string 116</String><String>string 117</String><String>string 118</String><String>string 119</String><String>string 120</String><String>string 121</String><String>string 122</String><String>string 123</String><String>string 124</String><String>string 125</String><String>string 126</String><String>string 127</String><String>string 128</String><String>string 129</String><String>string 130</String><String>string 131</String><String>string 132</String><String>string 133</String><String>string 134</String><String>string 135</String><String>string 136</String><String>string 137</String><String>string 138</String><String>string 139</String><String>string 140</String><String>string 141</String><String>string 142</String><String>string 143</String><String>string 144</String><String>string 145</String><String>string 146</String><String>string 147</String><String>string 148</String><String>string 149</String><String>string 150</String><String>string 151</String><String>string 152</String><String>string 153</String><String>string 154</String><String>string 155</String><String>string 156</String><String>string 157</String><String>string 158</String><String>string 159</String><String>string 160</String><String>string 161</String><String>string 162</String><String>string 163</String><String>string 164</String><String>string 165</String><String>string 166</String><String>string 167</String><String>string 168</String><String>string 169</String><String>string 170</String><String>string 171</String><String>string 172</String><String>string 173</String><String>string 174</String><String>string 175</String><String>string 176</String><String>string 177</String><String>string 178</String><String>string 179</String><String>string 180</String><String>string 181</String><String>string 182</String><String>string 183</String><String>string 184</String><String>string 185</String><String>string 186</String><String>string 187</String><String>string 188</String><String>string 189</String><String>string 190</String><String>string 191</String><String>string 192</String><String>string 193</String><String>string 194</String><String>string 195</String><String>string 196</String><String>string 197</String><String>string 198</String><String>string 199</String><String>string 200</String><String>string 201</String><String>string 202</String><String>string 203</String><String>string 204</String><String>string 205</String><String>string 206</String><String>string 207</String><String>string 208</String><String>string 209</String><String>string 210</String><String>string 211</String><String>string 212</String><String>string 213</String><String>string 214</String><String>string 215</String><String>string 216</String><String>string 217</String><String>string 218</String><String>string 219</String><String>string 220</String><String>string 221</String><String>string 222</String><String>string 223</String><String>string 224</String><String>string 225</String><String>string 226</String><String>string 227</String><String>string 228</String><String>string 229</String><String>string 230</String><String>string 231</String><String>string 232</String><String>string 233</String><String>string 234</String><String>string 235</String><String>string 236</String><String>string 237</String><String>string 238</String><String>string 239</String><String>string 240</String><String>string 241</String><String>string 242</String><String>string 243</String><String>string 244</String><String>string 245</String><String>string 246</String><String>string 247</String><String>string 248</String><String>string 249</String><String>string 250</String><String>string 251</String><String>string 252</String><String>string 253</String><String>string 254</String><String>string 255</String><String>string 256</String><String>string 257</String><String>string 258</String><String>string 259</String><String>string 260</String><String>string 261</String><String>string 262</String><String>string 263</String><String>string 264</String><String>string 265</String><String>string 266</String><String>string 267</String><String>string 268</String><String>string 269</String><String>string 270</String><String>string 271</String><String>string 272</String><String>string 273</String><String>string 274</String><String>string 275</String><String>string 276</String><String>string 277</String><String>string 278</String><String>string 279</String><String>string 280</String><String>string 281</String><String>string 282</String><String>string 283</String><String>string 284</String><String>string 285</String><String>string 286</String><String>string 287</String><String>string 288</String><String>string 289</String><String>string 290</String><String>string 291</String><String>string 292</String><String>string 293</String><String>string 294</String><String>string 295</String><String>string 296</String><String>string 297</String><String>string 298</String><String>string 299</String><String>string 300</String><String>string 301</String><String>string 302</String><String>string 303</String><String>string 304</String><String>string 305</String><String>string 306</String><String>string 307</String><String>string 308</String><String>string 309</String><String>string 310</String><String>string 311</String><String>string 312</String><String>string 313</String><String>string 314</String><String>string 315</String><String>string 316</String><String>string 317</String><String>string 318</String><String>string 319</String><String>string 320</String><String>string 321</String><String>string 322</String><String>string 323</String><String>string 324</String><String>string 325</String><String>string 326</String><String>string 327</String><String>string 328</String><String>string 329</String><String>string 330</String><String>string 331</String><String>string 332</String><String>string 333</String><String>string 334</String><String>string 335</String><String>string 336</String><String>string 337</String><String>string 338</String><String>string 339</String><String>string 340</String><String>string 341</String><String>string 342</String><String>string 343</String><String>string 344</String><String>string 345</String><String>string 346</String><String>string 347</String><String>string 348</String><String>string 349</String><String>string 350</String><String>string 351</String><String>string 352</String><String>string 353</String><String>string 354</String><String>string 355</String><String>string 356</String><String>string 357</String><String>string 358</String><String>string 359</String><String>string 360</String><String>string 361</String><String>string 362</String><String>string 363</String><String>string 364</String><String>string 365</String><String>string 366</String><String>string 367</String><String>string 368</String><String>string 369</String><String>string 370</String><String>string 371</String><String>string 372</String><String>string 373</String><String>string 374</String><String>string 375</String><String>string 376</String><String>string 377</String><String>string 378</String><String>string 379</String><String>string 380</String><String>string 381</String><String>string 382</String><String>string 383</String><String>string 384</String><String>string 385</String><String>string 386</String><String>string 387</String><String>string 388</String><String>string 389</String><String>string 390</String><String>string 391</String><String>string 392</String><String>string 393</String><String>string 394</String><String>string 395</String><String>string 396</String><String>string 397</String><String>string 398</String><String>string 399</String><String>string 400</String><String>string 401</String><String>string 402</String><String>string 403</String><String>string 404</String><String>string 405</String><String>string 406</String><String>string 407</String><String>string 408</String><String>string 409</String><String>string 410</String><String>string 411</String><String>string 412</String><String>string 413</String><String>string 414</String><String>string 415</String><String>string 416</String><String>string 417</String><String>string 418</String><String>string 419</String><String>string 420</String><String>string 421</String><String>string 422</String><String>string 423</String><String>string 424</String><String>string 425</String><String>string 426</String><String>string 427</String><String>string 428</String><String>string 429</String><String>string 430</String><String>string 431</String><String>string 432</String><String>string 433</String><String>string 434</String><String>string 435</String><String>string 436</String><String>string 437</String><String>string 438</String><String>string 439</String><String>string 440</String><String>string 441</String><String>string 442</String><String>string 443</String><String>string 444</String><String>string 445</String><String>string 446</String><String>string 447</String><String>string 448</String><String>string 449</String><String>string 450</String><String>string 451</String><String>string 452</String><String>string 453</String><String>string 454</String><String>string 455</String><String>string 456</String><String>string 457</String><String>string 458</String><String>string 459</String><String>string 460</String><String>string 461</String><String>string 462</String><String>string 463</String><String>string 464</String><String>string 465</String><String>string 466</String><String>string 467</String><String>string 468</String><String>string 469</String><String>string 470</String><String>string 471</String><String>string 472</String><String>string 473</String><String>string 474</String><String>string 475</String><String>string 476</String><String>string 477</String><String>string 478</String><String>string 479</String><String>string 480</String><String>string 481</String><String>string 482</String><String>string 483</String><String>string 484</String><String>string 485</String><String>string 486</String><String>string 487</String><String>string 488</String><String>string 489</String><String>string 490</String><String>string 491</String><String>string 492</String><String>string 493</String><String>string 494</String><String>string 495</String><String>string 496</String><String>string 497</String><String>string 498</String><String>string 499</String><String>string 500</String></simpleSet></message>".getBytes())));
        // Get current time for getting log entries later

        Timestamp getTimeAsTimeStamp7 = new Timestamp(System.currentTimeMillis());
        // Make 4 REST calls to the operation
        cougarManager1.makeRestCougarHTTPCalls(getNewHttpCallBean1);
        // Create the expected response as an XML document
        XMLHelpers xMLHelpers3 = new XMLHelpers();
        Document createAsDocument9 = xMLHelpers3.getXMLObjectFromString("<SimpleSetOperationResponseObject><responseSet><String>string 001</String><String>string 002</String><String>string 003</String><String>string 004</String><String>string 005</String><String>string 006</String><String>string 007</String><String>string 008</String><String>string 009</String><String>string 010</String><String>string 011</String><String>string 012</String><String>string 013</String><String>string 014</String><String>string 015</String><String>string 016</String><String>string 017</String><String>string 018</String><String>string 019</String><String>string 020</String><String>string 021</String><String>string 022</String><String>string 023</String><String>string 024</String><String>string 025</String><String>string 026</String><String>string 027</String><String>string 028</String><String>string 029</String><String>string 030</String><String>string 031</String><String>string 032</String><String>string 033</String><String>string 034</String><String>string 035</String><String>string 036</String><String>string 037</String><String>string 038</String><String>string 039</String><String>string 040</String><String>string 041</String><String>string 042</String><String>string 043</String><String>string 044</String><String>string 045</String><String>string 046</String><String>string 047</String><String>string 048</String><String>string 049</String><String>string 050</String><String>string 051</String><String>string 052</String><String>string 053</String><String>string 054</String><String>string 055</String><String>string 056</String><String>string 057</String><String>string 058</String><String>string 059</String><String>string 060</String><String>string 061</String><String>string 062</String><String>string 063</String><String>string 064</String><String>string 065</String><String>string 066</String><String>string 067</String><String>string 068</String><String>string 069</String><String>string 070</String><String>string 071</String><String>string 072</String><String>string 073</String><String>string 074</String><String>string 075</String><String>string 076</String><String>string 077</String><String>string 078</String><String>string 079</String><String>string 080</String><String>string 081</String><String>string 082</String><String>string 083</String><String>string 084</String><String>string 085</String><String>string 086</String><String>string 087</String><String>string 088</String><String>string 089</String><String>string 090</String><String>string 091</String><String>string 092</String><String>string 093</String><String>string 094</String><String>string 095</String><String>string 096</String><String>string 097</String><String>string 098</String><String>string 099</String><String>string 100</String><String>string 101</String><String>string 102</String><String>string 103</String><String>string 104</String><String>string 105</String><String>string 106</String><String>string 107</String><String>string 108</String><String>string 109</String><String>string 110</String><String>string 111</String><String>string 112</String><String>string 113</String><String>string 114</String><String>string 115</String><String>string 116</String><String>string 117</String><String>string 118</String><String>string 119</String><String>string 120</String><String>string 121</String><String>string 122</String><String>string 123</String><String>string 124</String><String>string 125</String><String>string 126</String><String>string 127</String><String>string 128</String><String>string 129</String><String>string 130</String><String>string 131</String><String>string 132</String><String>string 133</String><String>string 134</String><String>string 135</String><String>string 136</String><String>string 137</String><String>string 138</String><String>string 139</String><String>string 140</String><String>string 141</String><String>string 142</String><String>string 143</String><String>string 144</String><String>string 145</String><String>string 146</String><String>string 147</String><String>string 148</String><String>string 149</String><String>string 150</String><String>string 151</String><String>string 152</String><String>string 153</String><String>string 154</String><String>string 155</String><String>string 156</String><String>string 157</String><String>string 158</String><String>string 159</String><String>string 160</String><String>string 161</String><String>string 162</String><String>string 163</String><String>string 164</String><String>string 165</String><String>string 166</String><String>string 167</String><String>string 168</String><String>string 169</String><String>string 170</String><String>string 171</String><String>string 172</String><String>string 173</String><String>string 174</String><String>string 175</String><String>string 176</String><String>string 177</String><String>string 178</String><String>string 179</String><String>string 180</String><String>string 181</String><String>string 182</String><String>string 183</String><String>string 184</String><String>string 185</String><String>string 186</String><String>string 187</String><String>string 188</String><String>string 189</String><String>string 190</String><String>string 191</String><String>string 192</String><String>string 193</String><String>string 194</String><String>string 195</String><String>string 196</String><String>string 197</String><String>string 198</String><String>string 199</String><String>string 200</String><String>string 201</String><String>string 202</String><String>string 203</String><String>string 204</String><String>string 205</String><String>string 206</String><String>string 207</String><String>string 208</String><String>string 209</String><String>string 210</String><String>string 211</String><String>string 212</String><String>string 213</String><String>string 214</String><String>string 215</String><String>string 216</String><String>string 217</String><String>string 218</String><String>string 219</String><String>string 220</String><String>string 221</String><String>string 222</String><String>string 223</String><String>string 224</String><String>string 225</String><String>string 226</String><String>string 227</String><String>string 228</String><String>string 229</String><String>string 230</String><String>string 231</String><String>string 232</String><String>string 233</String><String>string 234</String><String>string 235</String><String>string 236</String><String>string 237</String><String>string 238</String><String>string 239</String><String>string 240</String><String>string 241</String><String>string 242</String><String>string 243</String><String>string 244</String><String>string 245</String><String>string 246</String><String>string 247</String><String>string 248</String><String>string 249</String><String>string 250</String><String>string 251</String><String>string 252</String><String>string 253</String><String>string 254</String><String>string 255</String><String>string 256</String><String>string 257</String><String>string 258</String><String>string 259</String><String>string 260</String><String>string 261</String><String>string 262</String><String>string 263</String><String>string 264</String><String>string 265</String><String>string 266</String><String>string 267</String><String>string 268</String><String>string 269</String><String>string 270</String><String>string 271</String><String>string 272</String><String>string 273</String><String>string 274</String><String>string 275</String><String>string 276</String><String>string 277</String><String>string 278</String><String>string 279</String><String>string 280</String><String>string 281</String><String>string 282</String><String>string 283</String><String>string 284</String><String>string 285</String><String>string 286</String><String>string 287</String><String>string 288</String><String>string 289</String><String>string 290</String><String>string 291</String><String>string 292</String><String>string 293</String><String>string 294</String><String>string 295</String><String>string 296</String><String>string 297</String><String>string 298</String><String>string 299</String><String>string 300</String><String>string 301</String><String>string 302</String><String>string 303</String><String>string 304</String><String>string 305</String><String>string 306</String><String>string 307</String><String>string 308</String><String>string 309</String><String>string 310</String><String>string 311</String><String>string 312</String><String>string 313</String><String>string 314</String><String>string 315</String><String>string 316</String><String>string 317</String><String>string 318</String><String>string 319</String><String>string 320</String><String>string 321</String><String>string 322</String><String>string 323</String><String>string 324</String><String>string 325</String><String>string 326</String><String>string 327</String><String>string 328</String><String>string 329</String><String>string 330</String><String>string 331</String><String>string 332</String><String>string 333</String><String>string 334</String><String>string 335</String><String>string 336</String><String>string 337</String><String>string 338</String><String>string 339</String><String>string 340</String><String>string 341</String><String>string 342</String><String>string 343</String><String>string 344</String><String>string 345</String><String>string 346</String><String>string 347</String><String>string 348</String><String>string 349</String><String>string 350</String><String>string 351</String><String>string 352</String><String>string 353</String><String>string 354</String><String>string 355</String><String>string 356</String><String>string 357</String><String>string 358</String><String>string 359</String><String>string 360</String><String>string 361</String><String>string 362</String><String>string 363</String><String>string 364</String><String>string 365</String><String>string 366</String><String>string 367</String><String>string 368</String><String>string 369</String><String>string 370</String><String>string 371</String><String>string 372</String><String>string 373</String><String>string 374</String><String>string 375</String><String>string 376</String><String>string 377</String><String>string 378</String><String>string 379</String><String>string 380</String><String>string 381</String><String>string 382</String><String>string 383</String><String>string 384</String><String>string 385</String><String>string 386</String><String>string 387</String><String>string 388</String><String>string 389</String><String>string 390</String><String>string 391</String><String>string 392</String><String>string 393</String><String>string 394</String><String>string 395</String><String>string 396</String><String>string 397</String><String>string 398</String><String>string 399</String><String>string 400</String><String>string 401</String><String>string 402</String><String>string 403</String><String>string 404</String><String>string 405</String><String>string 406</String><String>string 407</String><String>string 408</String><String>string 409</String><String>string 410</String><String>string 411</String><String>string 412</String><String>string 413</String><String>string 414</String><String>string 415</String><String>string 416</String><String>string 417</String><String>string 418</String><String>string 419</String><String>string 420</String><String>string 421</String><String>string 422</String><String>string 423</String><String>string 424</String><String>string 425</String><String>string 426</String><String>string 427</String><String>string 428</String><String>string 429</String><String>string 430</String><String>string 431</String><String>string 432</String><String>string 433</String><String>string 434</String><String>string 435</String><String>string 436</String><String>string 437</String><String>string 438</String><String>string 439</String><String>string 440</String><String>string 441</String><String>string 442</String><String>string 443</String><String>string 444</String><String>string 445</String><String>string 446</String><String>string 447</String><String>string 448</String><String>string 449</String><String>string 450</String><String>string 451</String><String>string 452</String><String>string 453</String><String>string 454</String><String>string 455</String><String>string 456</String><String>string 457</String><String>string 458</String><String>string 459</String><String>string 460</String><String>string 461</String><String>string 462</String><String>string 463</String><String>string 464</String><String>string 465</String><String>string 466</String><String>string 467</String><String>string 468</String><String>string 469</String><String>string 470</String><String>string 471</String><String>string 472</String><String>string 473</String><String>string 474</String><String>string 475</String><String>string 476</String><String>string 477</String><String>string 478</String><String>string 479</String><String>string 480</String><String>string 481</String><String>string 482</String><String>string 483</String><String>string 484</String><String>string 485</String><String>string 486</String><String>string 487</String><String>string 488</String><String>string 489</String><String>string 490</String><String>string 491</String><String>string 492</String><String>string 493</String><String>string 494</String><String>string 495</String><String>string 496</String><String>string 497</String><String>string 498</String><String>string 499</String><String>string 500</String></responseSet></SimpleSetOperationResponseObject>");
        // Convert the expected response to REST types for comparison with actual responses
        Map<CougarMessageProtocolRequestTypeEnum, Object> convertResponseToRestTypes10 = cougarManager1.convertResponseToRestTypes(createAsDocument9, getNewHttpCallBean1);
        // Check the 4 responses are as expected
        HttpResponseBean response4 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTXMLXML);
        AssertionUtils.multiAssertEquals(convertResponseToRestTypes10.get(CougarMessageProtocolRequestTypeEnum.RESTXML), response4.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response4.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response4.getHttpStatusText());
        
        HttpResponseBean response5 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTXMLJSON);
        AssertionUtils.multiAssertEquals(convertResponseToRestTypes10.get(CougarMessageProtocolRequestTypeEnum.RESTJSON), response5.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response5.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response5.getHttpStatusText());
        
        HttpResponseBean response6 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTJSONXML);
        AssertionUtils.multiAssertEquals(convertResponseToRestTypes10.get(CougarMessageProtocolRequestTypeEnum.RESTXML), response6.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response6.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response6.getHttpStatusText());
        
        HttpResponseBean response7 = getNewHttpCallBean1.getResponseObjectsByEnum(com.betfair.testing.utils.cougar.enums.CougarMessageProtocolResponseTypeEnum.RESTJSONJSON);
        AssertionUtils.multiAssertEquals(convertResponseToRestTypes10.get(CougarMessageProtocolRequestTypeEnum.RESTJSON), response7.getResponseObject());
        AssertionUtils.multiAssertEquals((int) 200, response7.getHttpStatusCode());
        AssertionUtils.multiAssertEquals("OK", response7.getHttpStatusText());
        
        // generalHelpers.pauseTest(500L);
        // Check the log entries are as expected
        
        cougarManager1.verifyRequestLogEntriesAfterDate(getTimeAsTimeStamp7, new RequestLogRequirement("2.8", "simpleSetOperation"),new RequestLogRequirement("2.8", "simpleSetOperation"),new RequestLogRequirement("2.8", "simpleSetOperation"),new RequestLogRequirement("2.8", "simpleSetOperation") );
    }

}
